

import logging

import pygame


log = logging.getLogger('Ascii')


class AsciiSheet(object):
    def __init__(self, sheet_filename, stride, color=None, background=None):
        self._sheet_file = pygame.image.load(sheet_filename)
        self._sheet_file.set_colorkey((255, 0, 255))
        self._sheet_file = self._sheet_file.convert_alpha()
        self._background = background

        w = self._sheet_file.get_width()
        h = self._sheet_file.get_height()

        self._sheet = pygame.surface.Surface((w, h), pygame.SRCALPHA)
        self._sheet.blit(self._sheet_file, (0, 0, 0, 0))

        self._stride = stride
        self._rect = self._sheet.get_rect()
        self._w = stride
        self._h = 256 / self._stride
        self._tw = self._rect.w / self._w
        self._th = self._rect.h / self._h

        if color is not None:
            self._sheet.fill(color, None, pygame.BLEND_RGB_MIN)

        if background is not None:
            temp_sheet = pygame.surface.Surface((w, h))
            temp_sheet.fill(background)
            temp_sheet.blit(self._sheet, (0, 0, 0, 0))
            self._sheet = temp_sheet

        log.info('Created sheet with fg:{} / bg:{}'.format(color, background))

    def _get_blit_rect(self, source, tile):
        x = (tile % self._stride) * self._tw
        y = (tile / self._stride) * self._th
        return (x, y, self._tw, self._th)

    def draw_tile(self, dest, x, y, tile):
        x *= self._tw
        y *= self._th
        blit_rect = self._get_blit_rect(self._sheet, tile)
        dest_rect = (x, y, self._tw, self._th)
        rect = dest.blit(self._sheet, dest_rect, blit_rect)
        return rect

    def draw_window(self, dest, w, h):
        """
        [TL, T, TR, R, BR, B, BL]
        """
        tw, th = self._tw, self._th
        wts = [201, 205, 187, 186, 188, 205, 200]
        for y in range(1, h-1):
            for x in range(1, w-1):
                self.draw_tile(dest, x, y, 0)
        # TL
        self.draw_tile(dest, 0, 0, wts[0])
        # TR
        self.draw_tile(dest, w-1, 0, wts[2])
        # BR
        self.draw_tile(dest, w-1, h-1, wts[4])
        # BL
        self.draw_tile(dest, 0, h-1, wts[6])
        # T + B
        for n in range(1, w-1):
            self.draw_tile(dest, n, 0, wts[1])
            self.draw_tile(dest, n, h-1, wts[1])
        for n in range(1, h-1):
            self.draw_tile(dest, 0, n, wts[3])
            self.draw_tile(dest, w-1, n, wts[3])
        return (0, 0, w*tw, h*th)
