import logging
from random import randint
import json

import pygame

from window import Window
from state import GameState
from ascii import AsciiSheet
from scene import Scene
from texts import Texts
import stats

logging.basicConfig(level=logging.INFO)


class BattleState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(BattleState, self).__init__(
            window, 'BattleState', *args, **kwargs)
        self.messages = json.loads(open('assets/strings/monsters.json').read())
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer(kwargs['backdrop'])
        self.tl = self.scene.add_text_layer()
        self.monster_hp = 10
        self.player_hp = 10
        self.speed = kwargs['speed']
        self.monster_name = kwargs['monster_name']
        self.next_state = kwargs['next_state']
        self.mode = 'attack'
        self.pp_layer = None
        self.last_status = None
        self.update_combat_layers()


    def update_combat_layers(self):
        self.scene.remove_layer(self.tl)
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, 'You are locked in combat with a {}!'.format(self.monster_name))
        self.tl.add_text(1, 11, 'You ({}/10)'.format(self.player_hp))
        self.tl.add_text(1, 12, '{} ({}/10)'.format(self.monster_name, self.monster_hp))
        if self.mode == 'attack':
            self.tl.add_text(30, 6, 'Attacking a {}'.format(self.monster_name))
            self.pp_layer = self.scene.add_ping_pong_layer(30, 8, 30, 4, speed=self.speed)
        elif self.mode == 'post attack':
            self.tl.add_text(30, 6, self.messages[self.monster_name][self.last_status].format(name=self.monster_name))
            self.tl.add_text(50, 13, 'Press space to continue...')
        elif self.mode == 'dodge':
            self.tl.add_text(30, 6, 'Dodging a {}'.format(self.monster_name))
            self.pp_layer = self.scene.add_ping_pong_layer(30, 8, 30, 4, speed=self.speed)
        elif self.mode == 'post dodge':
            self.tl.add_text(30, 6, self.messages[self.monster_name][self.last_status].format(name=self.monster_name))
            self.tl.add_text(50, 13, 'Press space to continue...')
        elif self.mode == 'monster killed':
            self.tl.add_text(30, 6, self.messages[self.monster_name]['death'].format(name=self.monster_name))
            self.tl.add_text(50, 13, 'Press space to continue...')
        self.scene.redraw_all()

    def check_player_death(self):
        if self.player_hp <= 0:
            self.window.pop_state()
            self.window.push_state(DefeatState)
            return True
        return False


    def on_keydown(self, event):
        if event.key == pygame.K_s:
            self.window.pop_state()
            self.window.push_state(self.next_state)

        if event.key == pygame.K_SPACE:
            if self.mode == 'attack':
                if self.pp_layer.check_hit() is True:
                    stats.get_instance().track_hit()
                    self.monster_hp -= 2
                    self.last_status = 'hit'
                else:
                    stats.get_instance().track_miss()
                    self.last_status = 'miss'
                self.pp_layer.stop()
                if self.monster_hp > 0:
                    self.mode = 'post attack'
                else:
                    self.mode = 'monster killed'
                self.update_combat_layers()

            elif self.mode == 'post attack':
                self.scene.remove_layer(self.pp_layer)
                self.mode = 'dodge'
                self.update_combat_layers()

            elif self.mode == 'dodge':
                if self.pp_layer.check_hit() is False:
                    stats.get_instance().track_dodge_fail()
                    self.player_hp -= 2
                    if self.monster_name == 'Revenant':
                        self.player_hp -= 1
                    elif self.monster_name == 'Prometheus':
                        self.player_hp -= 2
                    self.last_status = 'dodge_fail'
                else:
                    stats.get_instance().track_dodge()
                    self.last_status = 'dodge'
                self.pp_layer.stop()
                self.mode = 'post dodge'
                self.update_combat_layers()

            elif self.mode == 'post dodge':
                if self.check_player_death():
                    return
                self.scene.remove_layer(self.pp_layer)
                self.mode = 'attack'
                self.update_combat_layers()

            elif self.mode == 'monster killed':
                if self.monster_name == 'Forest Giant':
                    stats.get_instance().track_defeated_forest_giant()
                elif self.monster_name == 'Revenant':
                    stats.get_instance().track_defeated_revenant()
                elif self.monster_name == 'Prometheus':
                    stats.get_instance().track_defeated_prometheus()
                self.window.pop_state()
                self.window.push_state(self.next_state)


class Caucuses(GameState):
    def __init__(self, window, *args, **kwargs):
        super(Caucuses, self).__init__(
            window, 'Caucuses', *args, **kwargs)
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/caucuses.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['mountain_intro'])
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'mountain_intro'

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'mountain_intro':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['mountain_suspense'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'mountain_suspense'
            elif self.current_scene == 'mountain_suspense':
                pygame.mixer.music.load('assets/music/musix-the-red-one.mod')
                pygame.mixer.music.play(-1)
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['mountain_ambush'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'mountain_ambush'
            elif self.current_scene == 'mountain_ambush':
                self.window.pop_state()
                self.window.push_state(BattleState, backdrop='assets/raw/caucuses.json', monster_name='Prometheus', next_state=VictoryState, speed=650)


class DeadPlains(GameState):
    def __init__(self, window, *args, **kwargs):
        super(DeadPlains, self).__init__(
            window, 'DeadPlains', *args, **kwargs)
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/dead_plains.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['dead_intro'])
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'dead_intro'

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'dead_intro':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['dead_ambush'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'dead_ambush'
            elif self.current_scene == 'dead_ambush':
                self.window.pop_state()
                self.window.push_state(BattleState, backdrop='assets/raw/dead_plains.json', monster_name='Revenant', next_state=Caucuses, speed=800)


class ForestState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(ForestState, self).__init__(
            window, 'ForestState', *args, **kwargs)
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/ancient_forest.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['forest_intro'])
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'forest_intro'

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'forest_intro':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['forest_ambush'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'forest_giant'
            elif self.current_scene == 'forest_giant':
                self.window.pop_state()
                self.window.push_state(BattleState, backdrop='assets/raw/ancient_forest.json', monster_name='Forest Giant', next_state=DeadPlains, speed=1000)


class DelphiState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(DelphiState, self).__init__(
            window, 'DelphiState', *args, **kwargs)
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/delphi_valley.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['delphi_intro'])
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'delphi_intro'
        stats.get_instance().start_quest()

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'delphi_intro':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['delphi_quest'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'delphi_quest'
            elif self.current_scene == 'delphi_quest':
                self.window.pop_state()
                self.window.push_state(ForestState)

        if event.key == pygame.K_UP:
            self.menu.prev()
        if event.key == pygame.K_DOWN:
            self.menu.next()


class VictoryState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(VictoryState, self).__init__(
            window, 'VictoryState', *args, **kwargs)
        stats.get_instance().end_quest()
        pygame.mixer.music.load('assets/music/musix-wild-perspective.mod')
        pygame.mixer.music.play(-1)
        self.scene = Scene(self.window)
        self.bg = self.scene.add_tiled_layer('assets/raw/caucuses.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['mountain_victory'])
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'mountain_victory'

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'mountain_victory':
                self.scene.remove_layer(self.tl)
                self.scene.remove_layer(self.bg)
                self.bg = self.scene.add_tiled_layer('assets/raw/delphi_valley.json')
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['delphi_victory'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'delphi_victory'
            elif self.current_scene == 'delphi_victory':
                self.window.pop_state()
                self.window.push_state(StatsState)


class DefeatState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(DefeatState, self).__init__(
            window, 'DefeatState', *args, **kwargs)
        stats.get_instance().end_quest()
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/delphi_valley.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, Texts['defeat'])
        self.tl.add_text(50, 13, 'Press space to continue...')

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            self.window.pop_state()
            self.window.push_state(StatsState)


class KnuckleState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(KnuckleState, self).__init__(
            window, 'KnuckleState', *args, **kwargs)
        self.scene = Scene(self.window)
        tl = self.scene.add_text_layer()
        tl.add_text(20, 12, 'Knuckledragger Games')
        tl.add_text(26, 14, 'presents')
        tl.add_text(19, 16, 'Retirement of Diomedes')
        self.menu = self.scene.add_menu_layer(56, 36, 15, 4)
        self.menu.add_menu_item('New Game', self.new_game)
        self.menu.add_menu_item('Quit', self.quit)

    def new_game(self):
        stats.get_instance().reset()
        self.window.push_state(DelphiState)

    def quit(self):
        self.window.pop_state()

    def on_keydown(self, event):
        if event.key == pygame.K_RETURN:
            self.menu.activate()
        if event.key == pygame.K_UP:
            self.menu.prev()
        if event.key == pygame.K_DOWN:
            self.menu.next()

    def on_state_gain_focus(self):
        pygame.mixer.music.load('assets/music/musix-after.mod')
        pygame.mixer.music.play(-1)
        self.window.screen.fill((0, 0, 0))
        self.scene.draw(self.window)
        self.scene.redraw_all()
        pygame.display.flip()


class StatsState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(StatsState, self).__init__(window, 'StatsState', *args, **kwargs)
        self.pause = False
        self.sheets = {}
        self.scene = Scene(self.window)
        self.bg = self.scene.add_tiled_layer('assets/raw/ancient_forest.json')
        self.tl = self.scene.add_text_layer()
        self.tl.add_text(1, 1, stats.get_instance().get_combat_stats())
        self.tl.add_text(50, 13, 'Press space to continue...')
        self.current_scene = 'stats_combat'

    def on_keydown(self, event):
        if event.key == pygame.K_SPACE:
            if self.current_scene == 'stats_combat':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, stats.get_instance().get_goal_stats())
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'stats_goals'
            elif self.current_scene == 'stats_goals':
                self.scene.remove_layer(self.tl)
                self.tl = self.scene.add_text_layer()
                self.tl.add_text(1, 1, Texts['credits'])
                self.tl.add_text(50, 13, 'Press space to continue...')
                self.current_scene = 'stats_credits'
            elif self.current_scene == 'stats_credits':
                self.window.pop_state()

class TestState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(TestState, self).__init__(window, 'TestState', *args, **kwargs)
        self.pause = False
        self.sheets = {}
        self.scene = Scene(self.window)
        self.scene.add_tiled_layer('assets/raw/ancient_forest.json')
        tl = self.scene.add_text_layer()
        tl.add_text(1, 1, Texts['forest_intro'])
        self.debug = self.scene.add_text_layer()
        self.fps_string = self.debug.add_string(1, 44, '', 'menu')

        self.menu = self.scene.add_menu_layer(4, 25, 30, 16)
        self.menu.add_menu_item('FPS timer bump', self.menu_item_one)
        self.menu.add_menu_item('Second one', self.update_fps_label)
        self.menu.add_menu_item('Third one', self.update_fps_label)
        self.menu.add_menu_item('Close menu', self.close_menu)
        self.ppl = self.scene.add_ping_pong_layer(10, 10, 20, 8, 1500)

        self.window.scheduler.new_cycle_timer(1000, self.update_fps_label)

    def on_keydown(self, event):
        if event.key == pygame.K_ESCAPE:
            self.window.pop_state()
        if event.key == pygame.K_UP:
            self.menu.prev()
        if event.key == pygame.K_DOWN:
            self.menu.next()
        if event.key == pygame.K_RETURN:
            self.menu.activate()

    def on_draw(self):
        update_rects = []
        update_rects += self.scene.draw(self.window)
        # pygame.display.flip()
        # print len(update_rects)
        # print update_rects
        pygame.display.update(update_rects)

    def on_update(self, dt):
        pass

    def update_fps_label(self):
        self.fps_string.set_text('FPS: {}({})  '.format(
            self.window.fps, self.window.ups))

    def menu_item_one(self):
        print 'activated item 1'

    def close_menu(self):
        self.scene.remove_layer(self.menu)

    def on_state_lose_focus(self):
        self.ppl.stop()


class HitSliderState(GameState):
    def __init__(self, window, *args, **kwargs):
        super(HitSliderState, self).__init__(
            window, 'HitSliderState', *args, **kwargs)
        self.scene = Scene(self.window)
        self.ppl = self.scene.add_ping_pong_layer(10, 10, 20, 8, 1500)

    def on_keydown(self, event):
        if event.key == pygame.K_ESCAPE:
            self.window.pop_state()


if __name__ == '__main__':
    pygame.mixer.init()
    flags = 0  # pygame.FULLSCREEN
    window = Window(1280, 720, flags)
    window.push_state(KnuckleState, True)
    # import cProfile as profile
    # profile.run('window.run()')
    window.run()
