import stats
import time


class GameStats(object):
    def __init__(self):
        self.reset()

    def reset(self):
        self.quest_start_time = None
        self.quest_end_time = None
        self.hits = 0
        self.misses = 0
        self.crits = 0
        self.dodges = 0
        self.dodge_fails = 0
        self.defeated_forest_giant = False
        self.defeated_revenant = False
        self.defeated_prometheus = False

    def start_quest(self):
        self.quest_start_time = time.time()

    def end_quest(self):
        assert self.quest_start_time is not None
        self.quest_end_time = time.time()

    def get_quest_time(self):
        if self.quest_start_time is None:
            return 0
        elif self.quest_end_time is None:
            t = time.time()
        else:
            t = self.quest_end_time

        return t - self.quest_start_time

    def track_hit(self):
        self.hits += 1

    def track_miss(self):
        self.misses += 1

    def track_crit(self):
        self.crits += 1

    def track_dodge(self):
        self.dodges += 1

    def track_dodge_fail(self):
        self.dodge_fails += 1

    def track_defeated_forest_giant(self):
        self.defeated_forest_giant = True

    def track_defeated_revenant(self):
        self.defeated_revenant = True

    def track_defeated_prometheus(self):
        self.defeated_prometheus = True

    def get_combat_stats(self):
        info = {
            'hits': self.hits,
            'misses': self.misses,
            'attack_total': self.hits + self.misses,
            'dodges': self.dodges,
            'dodge_total': self.dodges + self.dodge_fails
        }
        return """Combat Stats

    You hit with {hits} out of your {attack_total} attacks.

    You dodged {dodges} out of {dodge_total} incoming attacks.""".format(**info)

    def get_goal_stats(self):
        s = "Goal Stats\n\n"
        if self.defeated_forest_giant:
            s += "You defeated the forest giant.\n"
        else:
            s += "You did not defeat the forest giant.\n"
        if self.defeated_revenant:
            s += "You defeated the revenant.\n"
        else:
            s += "You did not defeat the revenant.\n"
        if self.defeated_prometheus:
            s += "You defeated Prometheus.\n"
        else:
            s += "You did not defeat Prometheus.\n"

        t = round(self.get_quest_time(), 3)
        s += "\nGame time: {t} seconds".format(t=t)
        return s

instance_ = GameStats()

def get_instance():
    return instance_

